package corp.wattson.plinstool.client.ui;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashMap;

import javax.swing.BorderFactory;
import javax.swing.GroupLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.LayoutStyle;

import corp.wattson.plinstool.core.Describable;
import corp.wattson.plinstool.core.Describable.ProcessableState;

public class DescribablePanel extends JPanel {

  private static HashMap<ProcessableState, Color> colorMap = new HashMap<ProcessableState, Color>();
  static {
    colorMap.put(ProcessableState.NOT_CHECKED, Color.BLUE);
    colorMap.put(ProcessableState.BROKEN, Color.RED);
    colorMap.put(ProcessableState.FINE, Color.GREEN);
  }

  private Describable describable;
  private JTextArea description;
  private JButton checkButton;
  private JButton installButton;
  private JLabel stateLabel;

  public DescribablePanel(Describable describable) {
    this(describable, 0);
  }

  public DescribablePanel(Describable describable, int depth) {
    this.describable = describable;
    initComponents(depth);
  }

  private void initComponents(int depth) {
    setName(describable.getName() + "Panel");
    setBorder(BorderFactory.createTitledBorder(describable.getName()));
    GroupLayout ourPanelLayout = new GroupLayout(this);
    setLayout(ourPanelLayout);

    stateLabel = new JLabel();
    stateLabel.setFont(stateLabel.getFont().deriveFont(
        (stateLabel.getFont().getStyle() | java.awt.Font.ITALIC) | java.awt.Font.BOLD));
    stateLabel.setForeground(colorMap.get(describable.getState()));
    stateLabel.setText(describable.getState().toString());

    checkButton = new JButton();
    checkButton.setText("Check");
    checkButton.setName("checkButton");
    checkButton.addActionListener((new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        System.out.println("check clicked");
      }
    }));

    installButton = new JButton();
    installButton.setText("Install");
    installButton.setName("InstallButton");
    installButton.addActionListener((new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        System.out.println("install clicked");
      }
    }));

    description = new JTextArea();
    description.setBackground(new Color(240, 240, 240));
    description.setColumns(20);
    description.setEditable(false);
    description.setLineWrap(true);
    description.setRows(5);
    description.setWrapStyleWord(true);
    description.setBorder(null);
    description.setName("descriptionPanel");
    description.setText(describable.describe());

    ourPanelLayout.setHorizontalGroup(
        ourPanelLayout.createParallelGroup(GroupLayout.Alignment.LEADING).addGroup(
        ourPanelLayout.createSequentialGroup().addComponent(description, GroupLayout.PREFERRED_SIZE, 420,
            GroupLayout.PREFERRED_SIZE).addGap(18, 18, 18).addGroup(
            ourPanelLayout.createParallelGroup(GroupLayout.Alignment.LEADING, false).addComponent(stateLabel,
                GroupLayout.Alignment.TRAILING, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(
                    ourPanelLayout.createSequentialGroup().addComponent(checkButton).addGap(4, 4, 4).addComponent(
                        installButton)))));
    
    ourPanelLayout.setVerticalGroup(ourPanelLayout.createParallelGroup(GroupLayout.Alignment.LEADING).addGroup(
        ourPanelLayout.createSequentialGroup().addComponent(stateLabel).addPreferredGap(
            LayoutStyle.ComponentPlacement.RELATED).addGroup(
            ourPanelLayout.createParallelGroup(GroupLayout.Alignment.BASELINE).addComponent(installButton)
                .addComponent(checkButton, javax.swing.GroupLayout.PREFERRED_SIZE, 23, GroupLayout.PREFERRED_SIZE)))
        .addComponent(description, GroupLayout.PREFERRED_SIZE, 44, GroupLayout.PREFERRED_SIZE));
  }

}
