package corp.wattson.plinstool.core;

import java.util.List;

/**
 * This interface is used to describe an Item. It will be used by UI development.
 * 
 * @author wattazoum
 * 
 */
public interface Describable {

  /**
   * 
   * @author wattazoum
   * 
   */
  public enum ProcessableState {
    NOT_CHECKED("not checked"), FINE("fine"), BROKEN("broken");

    private String message;

    private ProcessableState(String message) {
      this.message = message;
    }

    public String getMessage() {
      return message;
    }

    public void setMessage(String message) {
      this.message = message;
    }
  }

  /**
   * Get a human readable name of the Describable.
   * 
   * @return The name of this {@link Describable}.
   */
  String getName();

  /**
   * Get a description of the Describable.
   * 
   * @return The asked description.
   */
  String describe();

  /**
   * Get the state of this {@link Describable}.
   * 
   * @return The state of this Describable.
   * @see ProcessableState
   */
  ProcessableState getState();

  /**
   * A describable can depend on others {@link Describable}. This method should return a list
   * (ordered or not) of the dependencies.
   * 
   * @return The list of describable this one includes.
   */
  List<Describable> getIncludedDescribable();
}
