package corp.wattson.plinstool.core;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.util.ArrayList;

import org.junit.Test;

import corp.wattson.plinstool.core.Describable.ProcessableState;
import corp.wattson.plinstool.core.Processable.NotCompliantInstallException;
import corp.wattson.plinstool.core.Processable.ProcessFailedException;

public class PlatformTest {

  @Test
  public void testEmptyPlatform() throws NotCompliantInstallException, ProcessFailedException {
    Platform platform = new Platform();
    platform.setName("Testing Platform");
    assertEquals("Testing Platform", platform.getName());
    platform.setDescription("My description");
    assertEquals("My description", platform.describe());
    assertNotNull(platform.getSoftwares());
    assertEquals(0, platform.getSoftwares().size());
    assertEquals(ProcessableState.NOT_CHECKED, platform.getState());
    platform.check();
    assertEquals(ProcessableState.FINE, platform.getState());
    platform.process();
    assertTrue(platform.getIncludedDescribable().isEmpty());
  }
  
  @Test
  public void testCheckPlatformWithFineItems() throws NotCompliantInstallException {
    ArrayList<SoftwareDefinition> software = new ArrayList<SoftwareDefinition>();
    ArrayList<AbstractItem> items = new ArrayList<AbstractItem>();
    items.add(new MockItems.CustomizableItem(Describable.ProcessableState.FINE));
    items.add(new MockItems.CustomizableItem(Describable.ProcessableState.FINE));
    items.add(new MockItems.CustomizableItem(Describable.ProcessableState.FINE));
    SoftwareDefinition s = new SoftwareDefinition();
    s.setItems(items);
    software.add(s);
    software.add(new SoftwareDefinition());
    Platform platform = new Platform();
    platform.setSoftwares(software);
    platform.check();
  }

  @Test(expected=NotCompliantInstallException.class)
  public void testCheckPlatformWithBrokenItems() throws NotCompliantInstallException {
    ArrayList<SoftwareDefinition> software = new ArrayList<SoftwareDefinition>();
    ArrayList<AbstractItem> items = new ArrayList<AbstractItem>();
    items.add(new MockItems.CustomizableItem(Describable.ProcessableState.FINE));
    items.add(new MockItems.FailsOnCheck());
    SoftwareDefinition s = new SoftwareDefinition();
    s.setItems(items);
    software.add(s);
    software.add(new SoftwareDefinition());
    Platform platform = new Platform();
    platform.setSoftwares(software);
    platform.check();
  }

  @Test
  public void testInstallPlatformWithFineItems() throws ProcessFailedException {
    ArrayList<SoftwareDefinition> software = new ArrayList<SoftwareDefinition>();
    ArrayList<AbstractItem> items = new ArrayList<AbstractItem>();
    items.add(new MockItems.CustomizableItem(Describable.ProcessableState.FINE));
    items.add(new MockItems.CustomizableItem(Describable.ProcessableState.FINE));
    items.add(new MockItems.CustomizableItem(Describable.ProcessableState.FINE));
    SoftwareDefinition s = new SoftwareDefinition();
    s.setItems(items);
    software.add(s);
    software.add(new SoftwareDefinition());
    Platform platform = new Platform();
    platform.setSoftwares(software);
    platform.process();
  }
  
  @Test(expected=ProcessFailedException.class)
  public void testInstallPlatformWithBrokenItems() throws ProcessFailedException {
    ArrayList<SoftwareDefinition> software = new ArrayList<SoftwareDefinition>();
    ArrayList<AbstractItem> items = new ArrayList<AbstractItem>();
    items.add(new MockItems.CustomizableItem(Describable.ProcessableState.FINE));
    items.add(new MockItems.FailsOnProcess());
    SoftwareDefinition s = new SoftwareDefinition();
    s.setItems(items);
    software.add(s);
    software.add(new SoftwareDefinition());
    Platform platform = new Platform();
    platform.setSoftwares(software);
    platform.process();
  }

}
