package corp.wattson.plinstool.core;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.fail;

import java.util.ArrayList;

import org.junit.Test;

import corp.wattson.plinstool.core.Processable.NotCompliantInstallException;
import corp.wattson.plinstool.core.Processable.ProcessFailedException;

public class SoftwareDefinitionTest {

  @Test
  public void testDescribe() {
    SoftwareDefinition s = new SoftwareDefinition();
    assertNull(s.describe());
    s.setDescription("My test description.");
    assertNotNull(s.describe());
    assertEquals("My test description.", s.describe());
    assertNull(s.getName());
    s.setName("testsoft");
    assertNotNull(s.getName());
    assertEquals("testsoft", s.getName());
  }

  @Test
  public void testCheck() throws NotCompliantInstallException {
    ArrayList<AbstractItem> items = new ArrayList<AbstractItem>();
    items.add(new MockItems.CustomizableItem(Describable.ProcessableState.FINE));
    SoftwareDefinition s = new SoftwareDefinition();
    s.setItems(items);

    SoftwareDefinition d = new SoftwareDefinition();
    d.setName("dependency");
    ArrayList<SoftwareDefinition> dependencies = new ArrayList<SoftwareDefinition>();
    dependencies.add(d);
    s.setDependencies(dependencies);

    s.check();
  }

  @Test
  public void testGetState() throws NotCompliantInstallException {
    SoftwareDefinition s = new SoftwareDefinition();
    assertEquals(Describable.ProcessableState.NOT_CHECKED, s.getState());
    s.check();
    assertEquals(Describable.ProcessableState.FINE, s.getState());
  }

  /**
   * Broken state cause of an Item.
   * 
   * @throws NotCompliantInstallException
   */
  @Test
  public void testGetBrokenStateOnItem() {
    ArrayList<AbstractItem> items = new ArrayList<AbstractItem>();
    items.add(new MockItems.FailsOnCheck());
    SoftwareDefinition s = new SoftwareDefinition();
    s.setItems(items);
    try {
      s.check();
      fail("We should have a NotCompliantInstallException !");
    }
    catch (NotCompliantInstallException e) {
      assertEquals(Describable.ProcessableState.BROKEN, s.getState());
    }
  }

  /**
   * Broken state cause of a dependency.
   * 
   * @throws NotCompliantInstallException
   */
  @Test
  public void testGetBrokenStateOnDep() {
    ArrayList<SoftwareDefinition> dependencies = new ArrayList<SoftwareDefinition>();
    dependencies.add(new FailingDep());
    SoftwareDefinition s = new SoftwareDefinition();
    s.setDependencies(dependencies);
    try {
      s.check();
      fail("We should have a NotCompliantInstallException !");
    }
    catch (NotCompliantInstallException e) {
      assertEquals(Describable.ProcessableState.BROKEN, s.getState());
    }
  }

  @Test
  public void testProcess() throws ProcessFailedException {
    ArrayList<AbstractItem> items = new ArrayList<AbstractItem>();
    items.add(new MockItems.CustomizableItem(Describable.ProcessableState.FINE));
    SoftwareDefinition s = new SoftwareDefinition();
    s.setItems(items);

    SoftwareDefinition d = new SoftwareDefinition();
    d.setName("dependency");
    ArrayList<SoftwareDefinition> dependencies = new ArrayList<SoftwareDefinition>();
    dependencies.add(d);
    s.setDependencies(dependencies);
    
    s.process();
  }

  private static class FailingDep extends SoftwareDefinition {
    @Override
    public void check() throws NotCompliantInstallException {
      throw new NotCompliantInstallException("Always thrown");
    }
  }

}
