package corp.wattson.plinstool.core;

import static org.junit.Assert.*;

import org.junit.After;
import org.junit.Test;

import corp.wattson.plinstool.core.ItemFactory.ItemCreationException;

public class TestItemFactory {

  @After
  public void tearDown() {
    ItemFactory.setCustomPackage(null);
  }

  @Test
  public void testGetValidItem() {
    ItemFactory.getItem("env");
    ItemFactory.getItem("Env");
  }

  @Test
  public void testGetMissingItem() {
    try {
      ItemFactory.getItem("missing");
      fail("This Item should be missing");
    }
    catch (ItemCreationException e) {
      assertEquals("Unable to find class for item type: missing", e.getMessage());
      assertTrue(e.getCause() instanceof ClassNotFoundException);
    }
    try {
      ItemFactory.getItem("eNV");
      fail("we shouldn't find class ENVItem !");
    }
    catch (ItemCreationException e) {
      assertEquals("Unable to find class: corp.wattson.plinstool.items.ENVItem", e.getMessage());
      assertTrue(e.getCause() instanceof ClassNotFoundException);
    }
    catch (NoClassDefFoundError e) {
      assertTrue("The class should not be found.", e.getMessage().contains(
          "corp/wattson/plinstool/items/common/ENVItem"));
    }
    ItemFactory.setCustomPackage("my.custom.items.pkg");
    try {
      ItemFactory.getItem("missing");
      fail("This Item should be missing");
    }
    catch (ItemCreationException e) {
      assertEquals("Unable to find class for item type: missing", e.getMessage());
      assertTrue(e.getCause() instanceof ClassNotFoundException);
    }
  }

  @Test
  public void testGetOSItem() {
    String osName = System.getProperty("os.name");
    osName = osName.split(" ")[0].toLowerCase();
    if ("windows".equals(osName)) {
      ItemFactory.getItem("exe");
    }
    else {
      fail("OS '" + osName + "' not supported!");
    }
  }

  @Test
  public void testGetCustomItem() {
    ItemFactory.setCustomPackage("my.custom.items.pkg");
    ItemFactory.getItem("custom");
  }

  @Test
  public void testIllegalArgs() {
    try {
      ItemFactory.getItem(null);
    }
    catch (IllegalArgumentException e) {
      assertEquals("The type shouldn't be null, nor empty !", e.getMessage());
    }
    try {
      ItemFactory.getItem("");
    }
    catch (IllegalArgumentException e) {
      assertEquals("The type shouldn't be null, nor empty !", e.getMessage());
    }
  }
  
  @Test
  public void testGetFalseItems() {
    ItemFactory.setCustomPackage("my.custom.items.pkg");
    try {
      ItemFactory.getItem("false");
    }
    catch (ItemCreationException e) {
      assertNotNull(e.getCause());
      assertTrue(e.getCause() instanceof ClassCastException);
    }
    try {
      ItemFactory.getItem("false2");
    }
    catch (ItemCreationException e) {
      assertNotNull(e.getCause());
      assertTrue(e.getCause() instanceof IllegalAccessException);
    }
    try {
      ItemFactory.getItem("false3");
    }
    catch (ItemCreationException e) {
      assertNotNull(e.getCause());
      assertTrue(e.getCause() instanceof InstantiationException);
    }
  }

}
